#--
# Mail-Retrainer, a spool manager for dovecot-antispam
# Copyright (c) 2012 Marc Dequènes (Duck) <Duck@DuckCorp.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#++

module MailRetrainer
    class MailFile
      attr_reader :filename, :signature, :arrival_ts, :user

      def initialize(filename, options)
        @filename = filename

        if options.file_pattern
          r = Regexp.new(options.file_pattern).match(File.basename(filename))
          if r.nil?
            raise "pattern does not match filename"
          end
          @user = r['user']
          @is_spam = (r['retrain_mode'] == "spam")
          @arrival_ts = r['arrival_ts'].to_i
        else
          filename =~ /.*\.(ham|spam)$/
          @is_spam = ($1 == "spam")

          @arrival_ts = Time.now.tv_sec
        end

        File.open(filename) do |io|
          io.each_line do |line|
            begin
              if line =~ /^#{options.signature_field}: (.+)$/
                @signature = $1
                break if @user
              elsif not @user and line =~ /^Delivered-To: (.+)$/
                @user = $1
                @user = @user.split('@', 2)[0] if options.strip_user_domain
                break if @signature
              end
            rescue
              # badly encoded file…
            end
          end
        end

        unless @signature
          raise "file without signature"
        end
        unless @user
          raise "file without user"
        end
        if @is_spam.nil?
          raise "cannot determine retrain mode in filename"
        end
      end

      def name
        "#{@signature}/#{@user}"
      end

      def spam?
        @is_spam
      end

      def delete
        File.delete(@filename)
      end
    end
end
